# !/usr/bin/env python3

# Anon
ANON = "ANON"
# Argoneum
AGM = "AGM"
# Artax
XAX = "XAX"
# Aryacoin
AYA = "AYA"
# Asiacoin
AC = "AC"
# Atom
ATOM = "ATOM"
# Auroracoin
AUR = "AUR"
# Axe
AXE = "AXE"
# Bata
BTA = "BTA"
# Beetle Coin
BEET = "BEET"
# Bela Coin
BELA = "BELA"
# Bit Cloud
BTDX = "BTDX"
# Bit Send
BSD = "BSD"
# Bitcoin Cash
BCH = "BCH"
# Bitcoin Gold
BTG = "BTG"
# Bitcoin
BTC, BTCTEST = "BTC", "BTCTEST"
# Bitcoin Plus
XBC = "XBC"
# Bitcoin SV
BSV = "BSV"
# BitcoinZ
BTCZ = "BTCZ"
# Bitcore
BTX = "BTX"
# Blackcoin
BLK = "BLK"
# Block Stamp
BST = "BST"
# Blocknode
BND, BNDTEST = "BND", "BNDTEST"
# Bolivarcoin
BOLI = "BOLI"
# Brit Coin
BRIT = "BRIT"
# CPU Chain
CPU = "CPU"
# Canada eCoin
CDN = "CDN"
# Cannacoin
CCN = "CCN"
# Clams
CLAM = "CLAM"
# Club Coin
CLUB = "CLUB"
# Compcoin
CMP = "CMP"
# Crane Pay
CRP = "CRP"
# Crave
CRAVE = "CRAVE"
# Dash
DASH, DASHTEST = "DASH", "DASHTEST"
# Deep Onion
ONION = "ONION"
# Defcoin
DFC = "DFC"
# Denarius
DNR = "DNR"
# Diamond
DMD = "DMD"
# Digi Byte
DGB = "DGB"
# Digitalcoin
DGC = "DGC"
# Dogecoin
DOGE, DOGETEST = "DOGE", "DOGETEST"
# EDR Coin
EDRC = "EDRC"
# Ecoin
ECN = "ECN"
# Einsteinium
EMC2 = "EMC2"
# Elastos
ELA = "ELA"
# Energi
NRG = "NRG"
# Ethereum
ETH = "ETH"
# Europe Coin
ERC = "ERC"
# Exclusive Coin
EXCL = "EXCL"
# FIX
FIX, FIXTEST = "FIX", "FIXTEST"
# Feathercoin
FTC = "FTC"
# Firstcoin
FRST = "FRST"
# Flashcoin
FLASH = "FLASH"
# Fuji Coin
FJC = "FJC"
# GCR Coin
GCR = "GCR"
# Game Credits
GAME = "GAME"
# Go Byte
GBX = "GBX"
# Gridcoin
GRC = "GRC"
# Groestl Coin
GRS, GRSTEST = "GRS", "GRSTEST"
# Gulden
NLG = "NLG"
# Helleniccoin
HNC = "HNC"
# Hempcoin
THC = "THC"
# Hush
HUSH = "HUSH"
# IX Coin
IXC = "IXC"
# Insane Coin
INSN = "INSN"
# Internet Of People
IOP = "IOP"
# Jumbucks
JBS = "JBS"
# Kobocoin
KOBO = "KOBO"
# Komodo
KMD = "KMD"
# LBRY Credits
LBC = "LBC"
# Linx
LINX = "LINX"
# Litecoin Cash
LCC = "LCC"
# Litecoin
LTC, LTCTEST = "LTC", "LTCTEST"
# LitecoinZ
LTZ = "LTZ"
# Lkrcoin
LKR = "LKR"
# Lynx
LYNX = "LYNX"
# Mazacoin
MZC = "MZC"
# Megacoin
MEC = "MEC"
# Minexcoin
MNX = "MNX"
# Monacoin
MONA = "MONA"
# Monkey Project
MONK = "MONK"
# Myriadcoin
XMY = "XMY"
# NIX
NIX = "NIX"
# Namecoin
NMC = "NMC"
# Navcoin
NAV = "NAV"
# Neblio
NEBL = "NEBL"
# Neoscoin
NEOS = "NEOS"
# Neurocoin
NRO = "NRO"
# New York Coin
NYC = "NYC"
# Novacoin
NVC = "NVC"
# NuBits
NBT = "NBT"
# NuShares
NSR = "NSR"
# OK Cash
OK = "OK"
# Omni
OMNI, OMNITEST = "OMNI", "OMNITEST"
# Onix Coin
ONX = "ONX"
# Peercoin
PPC = "PPC"
# Pesobit
PSB = "PSB"
# Phore
PHR = "PHR"
# Pinkcoin
PINK = "PINK"
# Pivx
PIVX, PIVXTEST = "PIVX", "PIVXTEST"
# Posw Coin
POSW = "POSW"
# Potcoin
POT = "POT"
# Project Coin
PRJ = "PRJ"
# Putincoin
PUT = "PUT"
# Qtum
QTUM, QTUMTEST = "QTUM", "QTUMTEST"
# RSK
RBTC, RBTCTEST = "RBTC", "RBTCTEST"
# Rapids
RPD = "RPD"
# Ravencoin
RVN = "RVN"
# Reddcoin
RDD = "RDD"
# Rubycoin
RBY = "RBY"
# Safecoin
SAFE = "SAFE"
# Saluscoin
SLS = "SLS"
# Scribe
SCRIBE = "SCRIBE"
# Shadow Cash
SDC, SDCTEST = "SDC", "SDCTEST"
# Slimcoin
SLM, SLMTEST = "SLM", "SLMTEST"
# Smileycoin
SMLY = "SMLY"
# Solarcoin
SLR = "SLR"
# Stash
STASH = "STASH"
# Stratis
STRAT, STRATTEST = "STRAT", "STRATTEST"
# Sugarchain
SUGAR, SUGARTEST = "SUGAR", "SUGARTEST"
# Syscoin
SYS = "SYS"
# TOA Coin
TOA = "TOA"
# Thought AI
THT = "THT"
# TRX
TRX = "TRX"
# Twins
TWINS, TWINSTEST = "TWINS", "TWINSTEST"
# Ultimate Secure Cash
USC = "USC"
# Unobtanium
UNO = "UNO"
# Virtual Cash
VASH = "VASH"
# Vcash
VC = "VC"
# Verge Currency
XVG = "XVG"
# Vertcoin
VTC = "VTC"
# Viacoin
VIA, VIATEST = "VIA", "VIATEST"
# Vivo
VIVO = "VIVO"
# Whitecoin
XWC = "XWC"
# Wincoin
WC = "WC"
# XUEZ
XUEZ = "XUEZ"
# XinFin
XDC = "XDC"
# ZClassic
ZCL = "ZCL"
# Zcash
ZEC, ZECTEST = "ZEC", "ZECTEST"
# Zencash
ZEN = "ZEN"


__all__ = [
    "ANON",
    "AGM",
    "XAX",
    "AYA",
    "AC",
    "ATOM",
    "AUR",
    "AXE",
    "BTA",
    "BEET",
    "BELA",
    "BTDX",
    "BSD",
    "BCH",
    "BTG",
    "BTC", "BTCTEST",
    "XBC",
    "BSV",
    "BTCZ",
    "BTX",
    "BLK",
    "BST",
    "BND", "BNDTEST",
    "BOLI",
    "BRIT",
    "CPU",
    "CDN",
    "CCN",
    "CLAM",
    "CLUB",
    "CMP",
    "CRP",
    "CRAVE",
    "DASH", "DASHTEST",
    "ONION",
    "DFC",
    "DNR",
    "DMD",
    "DGB",
    "DGC",
    "DOGE", "DOGETEST",
    "EDRC",
    "ECN",
    "EMC2",
    "ELA",
    "NRG",
    "ETH",
    "ERC",
    "EXCL",
    "FIX", "FIXTEST",
    "FTC",
    "FRST",
    "FLASH",
    "FJC",
    "GCR",
    "GAME",
    "GBX",
    "GRC",
    "GRS", "GRSTEST",
    "NLG",
    "HNC",
    "THC",
    "HUSH",
    "IXC",
    "INSN",
    "IOP",
    "JBS",
    "KOBO",
    "KMD",
    "LBC",
    "LINX",
    "LCC",
    "LTC", "LTCTEST",
    "LTZ",
    "LKR",
    "LYNX",
    "MZC",
    "MEC",
    "MNX",
    "MONA",
    "MONK",
    "XMY",
    "NIX",
    "NMC",
    "NAV",
    "NEBL",
    "NEOS",
    "NRO",
    "NYC",
    "NVC",
    "NBT",
    "NSR",
    "OK",
    "OMNI", "OMNITEST",
    "ONX",
    "PPC",
    "PSB",
    "PHR",
    "PINK",
    "PIVX", "PIVXTEST",
    "POSW",
    "POT",
    "PRJ",
    "PUT",
    "QTUM", "QTUMTEST",
    "RBTC", "RBTCTEST",
    "RPD",
    "RVN",
    "RDD",
    "RBY",
    "SAFE",
    "SLS",
    "SCRIBE",
    "SDC", "SDCTEST",
    "SLM", "SLMTEST",
    "SMLY",
    "SLR",
    "STASH",
    "STRAT", "STRATTEST",
    "SUGAR", "SUGARTEST",
    "SYS",
    "TOA",
    "THT",
    "TRX",
    "TWINS", "TWINSTEST",
    "USC",
    "UNO",
    "VASH",
    "VC",
    "XVG",
    "VTC",
    "VIA", "VIATEST",
    "VIVO",
    "XWC",
    "WC",
    "XUEZ",
    "XDC",
    "ZCL",
    "ZEC", "ZECTEST",
    "ZEN"
]
