# !/usr/bin/env python3

from hdwallet.symbols import *


def test_symbols():

    assert ANON == "ANON"
    assert AGM == "AGM"
    assert XAX == "XAX"
    assert AYA == "AYA"
    assert AC == "AC"
    assert ATOM == "ATOM"
    assert AUR == "AUR"
    assert AXE == "AXE"
    assert BTA == "BTA"
    assert BEET == "BEET"
    assert BELA == "BELA"
    assert BTDX == "BTDX"
    assert BSD == "BSD"
    assert BCH == "BCH"
    assert BTG == "BTG"
    assert BTC == "BTC"
    assert BTCTEST == "BTCTEST"
    assert XBC == "XBC"
    assert BSV == "BSV"
    assert BTCZ == "BTCZ"
    assert BTX == "BTX"
    assert BLK == "BLK"
    assert BST == "BST"
    assert BND == "BND"
    assert BNDTEST == "BNDTEST"
    assert BOLI == "BOLI"
    assert BRIT == "BRIT"
    assert CPU == "CPU"
    assert CDN == "CDN"
    assert CCN == "CCN"
    assert CLAM == "CLAM"
    assert CLUB == "CLUB"
    assert CMP == "CMP"
    assert CRP == "CRP"
    assert CRAVE == "CRAVE"
    assert DASH == "DASH"
    assert DASHTEST == "DASHTEST"
    assert ONION == "ONION"
    assert DFC == "DFC"
    assert DNR == "DNR"
    assert DMD == "DMD"
    assert DGB == "DGB"
    assert DGC == "DGC"
    assert DOGE == "DOGE"
    assert DOGETEST == "DOGETEST"
    assert EDRC == "EDRC"
    assert ECN == "ECN"
    assert EMC2 == "EMC2"
    assert ELA == "ELA"
    assert NRG == "NRG"
    assert ETH == "ETH"
    assert ERC == "ERC"
    assert EXCL == "EXCL"
    assert FIX == "FIX"
    assert FIXTEST == "FIXTEST"
    assert FTC == "FTC"
    assert FRST == "FRST"
    assert FLASH == "FLASH"
    assert FJC == "FJC"
    assert GCR == "GCR"
    assert GAME == "GAME"
    assert GBX == "GBX"
    assert GRC == "GRC"
    assert GRS == "GRS"
    assert GRSTEST == "GRSTEST"
    assert NLG == "NLG"
    assert HNC == "HNC"
    assert THC == "THC"
    assert HUSH == "HUSH"
    assert IXC == "IXC"
    assert INSN == "INSN"
    assert IOP == "IOP"
    assert JBS == "JBS"
    assert KOBO == "KOBO"
    assert KMD == "KMD"
    assert LBC == "LBC"
    assert LINX == "LINX"
    assert LCC == "LCC"
    assert LTC == "LTC"
    assert LTCTEST == "LTCTEST"
    assert LTZ == "LTZ"
    assert LKR == "LKR"
    assert LYNX == "LYNX"
    assert MZC == "MZC"
    assert MEC == "MEC"
    assert MNX == "MNX"
    assert MONA == "MONA"
    assert MONK == "MONK"
    assert XMY == "XMY"
    assert NIX == "NIX"
    assert NMC == "NMC"
    assert NAV == "NAV"
    assert NEBL == "NEBL"
    assert NEOS == "NEOS"
    assert NRO == "NRO"
    assert NYC == "NYC"
    assert NVC == "NVC"
    assert NBT == "NBT"
    assert NSR == "NSR"
    assert OK == "OK"
    assert OMNI == "OMNI"
    assert OMNITEST == "OMNITEST"
    assert ONX == "ONX"
    assert PPC == "PPC"
    assert PSB == "PSB"
    assert PHR == "PHR"
    assert PINK == "PINK"
    assert PIVX == "PIVX"
    assert PIVXTEST == "PIVXTEST"
    assert POSW == "POSW"
    assert POT == "POT"
    assert PRJ == "PRJ"
    assert PUT == "PUT"
    assert QTUM == "QTUM"
    assert QTUMTEST == "QTUMTEST"
    assert RBTC == "RBTC"
    assert RBTCTEST == "RBTCTEST"
    assert RPD == "RPD"
    assert RVN == "RVN"
    assert RDD == "RDD"
    assert RBY == "RBY"
    assert SAFE == "SAFE"
    assert SLS == "SLS"
    assert SCRIBE == "SCRIBE"
    assert SDC == "SDC"
    assert SDCTEST == "SDCTEST"
    assert SLM == "SLM"
    assert SLMTEST == "SLMTEST"
    assert SMLY == "SMLY"
    assert SLR == "SLR"
    assert STASH == "STASH"
    assert STRAT == "STRAT"
    assert STRATTEST == "STRATTEST"
    assert SUGAR == "SUGAR"
    assert SUGARTEST == "SUGARTEST"
    assert SYS == "SYS"
    assert TOA == "TOA"
    assert THT == "THT"
    assert TRX == "TRX"
    assert TWINS == "TWINS"
    assert TWINSTEST == "TWINSTEST"
    assert USC == "USC"
    assert UNO == "UNO"
    assert VASH == "VASH"
    assert VC == "VC"
    assert XVG == "XVG"
    assert VTC == "VTC"
    assert VIA == "VIA"
    assert VIATEST == "VIATEST"
    assert VIVO == "VIVO"
    assert XWC == "XWC"
    assert WC == "WC"
    assert XUEZ == "XUEZ"
    assert XDC == "XDC"
    assert ZCL == "ZCL"
    assert ZEC == "ZEC"
    assert ZECTEST == "ZECTEST"
    assert ZEN == "ZEN"
